#include "StdAfx.h"
#include "setup_window.h"
#include "dewesoft_bridge.h"
#include <thread>
#include <chrono>
#include <regex>

using namespace Dewesoft::MUI;
using namespace Dewesoft::RT::Core;

enum Cols
{
    ctNum,
    ctDesc,
};


SetupWindow::SetupWindow(WindowPtr ui, DewesoftBridge& bridge) : BaseSetupWindow(ui, bridge, "ui/setup_window.xml")
{
    uiRefreshTimer = Timer::Create(ui);
    uiRefreshTimer.setInterval(100);
    uiRefreshTimer.OnTimer += event(&SetupWindow::onUiRefreshTimer);

    channelGrid.setGridSize(3, 2);
    channelGrid.setColumn(ctNum, "#", ctNumber, RtTrue, 40, "Num");
    channelGrid.setColumn(ctDesc, "Description", ctEditText, RtTrue, 140, "Desc");
    channelGrid.getColumn(ctNum).setNumberFormat(cfInteger);
    channelGrid.applyColumns();
    channelGrid.OnCellGetProps += event(&SetupWindow::onGridGetProps);
    channelGrid.OnCellInput += event(&SetupWindow::onCellInputEventHandler);
    channelGrid.OnCellAction += event(&SetupWindow::onCellActionEventHandler);

    descriptions[0] = "Description";
    descriptions[1] = "Description";
}

void SetupWindow::onUiRefreshTimer(Dewesoft::MUI::Timer& ctrl, Dewesoft::MUI::EventArgs& args)
{
    if (channelGrid.assigned())
        channelGrid.invalidate();
}

void SetupWindow::onCellActionEventHandler(Dewesoft::MUI::DSDrawGrid& grid, Dewesoft::MUI::DrawGridCellActionEventArgs& args)
{
    int column = args.getColumn();
    int row = args.getRow();   
    if (column == ctDesc)
        descriptions[row - 1] = "Cell was pressed";
}

void SetupWindow::onCellInputEventHandler(Dewesoft::MUI::DSDrawGrid& grid, Dewesoft::MUI::DrawGridCellInputArgs& args)
{
    int column = args.getColumn();
    int row = args.getRow();
    if (column == ctDesc)
        descriptions[row - 1] = args.getText();
}


void SetupWindow::onGridGetProps(Dewesoft::MUI::DSDrawGrid& grid, Dewesoft::MUI::DrawGridCellPropsArgs& args)
{
    Int row = args.getRow();
    Int col = args.getColumn();

    switch (col)
    {
        case ctNum:
            args.setNumber(static_cast<Float>(row));
            break;        
        case ctDesc:            
            args.setText(descriptions[row - 1]);            
            break;        
        default:
            args.setText("");
            break;
    }
}
